@@ -106,6 +106,7 @@ oobs_ifaces_config_init (OobsIfacesConfig *config)
   priv->plip_ifaces = _oobs_list_new (OOBS_TYPE_IFACE_PLIP);
   priv->modem_ifaces = _oobs_list_new (OOBS_TYPE_IFACE_MODEM);
   priv->isdn_ifaces = _oobs_list_new (OOBS_TYPE_IFACE_ISDN);
+  config->_priv = priv;
 }
 
 static void
@@ -113,7 +114,7 @@ free_configuration (OobsIfacesConfig *config)
 {
   OobsIfacesConfigPrivate *priv;
 
-  priv = OOBS_IFACES_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   oobs_list_clear (priv->ethernet_ifaces);
   oobs_list_clear (priv->wireless_ifaces);
@@ -128,7 +129,7 @@ oobs_ifaces_config_finalize (GObject *object)
 {
   OobsIfacesConfigPrivate *priv;
 
-  priv = OOBS_IFACES_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_IFACES_CONFIG (object)->_priv;
 
   if (priv)
     {
@@ -185,7 +186,7 @@ create_iface_from_message (DBusMessage     *message,
 
   if (OOBS_IS_IFACE_ETHERNET (iface))
     {
-      const gchar *address, *netmask;
+      const gchar *address, *netmask, *gateway;
       gint config_method;
 
       dbus_message_iter_get_basic (&struct_iter, &is_auto);
@@ -207,11 +208,15 @@ create_iface_from_message (DBusMessage     *message,
       dbus_message_iter_next (&struct_iter);
       dbus_message_iter_next (&struct_iter);
 
+      gateway = utils_get_string (&struct_iter);
+      dbus_message_iter_next (&struct_iter);
+
       g_object_set (iface,
 		    "auto", is_auto,
 		    "active", active,
 		    "ip-address", address,
 		    "ip-mask", netmask,
+		    "gateway-address", gateway,
 		    /* FIXME: can pass config_method like that? */
 		    "config-method", config_method,
 		    NULL);
@@ -364,7 +369,7 @@ oobs_ifaces_config_update (OobsObject *object)
   DBusMessage *reply;
   DBusMessageIter iter;
 
-  priv = OOBS_IFACES_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_IFACES_CONFIG (object)->_priv;
   reply = _oobs_object_get_dbus_message (object);
 
   /* First of all, free the previous configuration */
@@ -413,12 +418,13 @@ create_dbus_struct_from_iface (DBusMessage     *message,
 
   if (OOBS_IS_IFACE_ETHERNET (iface))
     {
-      gchar *address, *netmask;
+      gchar *address, *netmask, *gateway;
       gint config_method;
 
       g_object_get (G_OBJECT (iface),
 		    "ip-address", &address,
 		    "ip-mask", &netmask,
+		    "gateway-address", &gateway,
 		    "config-method", &config_method,
 		    NULL);
 
@@ -431,7 +437,7 @@ create_dbus_struct_from_iface (DBusMessage     *message,
       utils_append_string (&iter, NULL);
       utils_append_string (&iter, NULL);
 
-      /* FIXME: missing gateway and possibly other stuff */
+      utils_append_string (&iter, (configured) ? gateway : NULL);
 
       if (OOBS_IS_IFACE_WIRELESS (iface))
 	{
@@ -454,6 +460,7 @@ create_dbus_struct_from_iface (DBusMessage     *message,
 
       g_free (address);
       g_free (netmask);
+      g_free (gateway);
     }
   else if (OOBS_IS_IFACE_PLIP (iface))
     {
@@ -551,6 +558,7 @@ create_dbus_struct_from_ifaces_list (OobsObject      *object,
 	DBUS_TYPE_STRING_AS_STRING
 	DBUS_TYPE_STRING_AS_STRING
 	DBUS_TYPE_STRING_AS_STRING
+	DBUS_TYPE_STRING_AS_STRING
 	DBUS_STRUCT_END_CHAR_AS_STRING;
       break;
     case OOBS_IFACE_TYPE_WIRELESS:
@@ -565,6 +573,7 @@ create_dbus_struct_from_ifaces_list (OobsObject      *object,
 	DBUS_TYPE_STRING_AS_STRING
 	DBUS_TYPE_STRING_AS_STRING
 	DBUS_TYPE_STRING_AS_STRING
+	DBUS_TYPE_STRING_AS_STRING
 	DBUS_TYPE_INT32_AS_STRING
 	DBUS_TYPE_STRING_AS_STRING
 	DBUS_STRUCT_END_CHAR_AS_STRING;
@@ -643,7 +652,7 @@ oobs_ifaces_config_commit (OobsObject *object)
   DBusMessage *message;
   DBusMessageIter iter;
 
-  priv = OOBS_IFACES_CONFIG_GET_PRIVATE (object);
+  priv = OOBS_IFACES_CONFIG (object)->_priv;
   message = _oobs_object_get_dbus_message (object);
 
   dbus_message_iter_init_append (message, &iter);
@@ -656,6 +665,15 @@ oobs_ifaces_config_commit (OobsObject *object)
   create_dbus_struct_from_ifaces_list (object, message, &iter, priv->isdn_ifaces, OOBS_IFACE_TYPE_ISDN);
 }
 
+/**
+ * oobs_ifaces_config_get:
+ * @session: An #OobsSession.
+ * 
+ * Returns the #OobsIfacesConfig singleton, which represents
+ * the network interfaces and their configuration.
+ * 
+ * Return Value: the singleton #OobsIfacesConfig object.
+ **/
 OobsObject*
 oobs_ifaces_config_get (OobsSession *session)
 {
@@ -676,6 +694,16 @@ oobs_ifaces_config_get (OobsSession *session)
   return object;
 }
 
+/**
+ * oobs_ifaces_config_get_ifaces:
+ * @config: An #OobsIfacesConfig.
+ * @type: An #OobsIfaceType.
+ * 
+ * Returns an #OobsList containing the interfaces that match the
+ * type defined by @type.
+ * 
+ * Return Value: An #OobsList, you must not unref this object.
+ **/
 OobsList*
 oobs_ifaces_config_get_ifaces (OobsIfacesConfig *config,
 			       OobsIfaceType     type)
@@ -684,7 +712,7 @@ oobs_ifaces_config_get_ifaces (OobsIfacesConfig *config,
 
   g_return_val_if_fail (OOBS_IS_IFACES_CONFIG (config), NULL);
 
-  priv = OOBS_IFACES_CONFIG_GET_PRIVATE (config);
+  priv = config->_priv;
 
   switch (type)
     {
